summaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorGerrit <chrome-bot@google.com>2012-05-29 10:12:37 -0700
committerGerrit Code Review <gerrit@gerrit.golo.chromium.org>2012-05-29 10:12:37 -0700
commit7f66786b76464298a8fc92b2b2377d3652fda914 (patch)
treecfbe7073d4bd94423c859767f9698a1b1ca0aaf3
parentcbd18dbe5b081abff2779993aa273f9bed7c2f2a (diff)
parentef46903dea21dee8932b5413c1c0ae6cd3a4eb16 (diff)
downloadchrome-ec-1.1.0.tar.gz
Merge "usleep() chains to udelay() if called before task_start()"v1.1.0
-rw-r--r--core/cortex-m/task.c9
-rw-r--r--core/cortex-m/timer.c7
-rw-r--r--include/task.h16
3 files changed, 26 insertions, 6 deletions
diff --git a/core/cortex-m/task.c b/core/cortex-m/task.c
index aaaa776385..41b0b7e39a 100644
--- a/core/cortex-m/task.c
+++ b/core/cortex-m/task.c
@@ -129,6 +129,8 @@ static int need_resched_or_profiling = 0;
*/
static uint32_t tasks_ready = (1<<TASK_ID_COUNT) - 1;
+static int start_called; /* Has task swapping started */
+
static task_ *__get_current(void)
{
@@ -211,6 +213,12 @@ uint32_t *task_get_event_bitmap(task_id_t tskid)
}
+int task_start_called(void)
+{
+ return start_called;
+}
+
+
/* Scheduling system call */
void svc_handler(int desched, task_id_t resched)
{
@@ -597,6 +605,7 @@ int task_start(void)
#ifdef CONFIG_TASK_PROFILING
task_start_time = exc_end_time = get_time().val;
#endif
+ start_called = 1;
return __task_start(&need_resched_or_profiling);
}
diff --git a/core/cortex-m/timer.c b/core/cortex-m/timer.c
index 14e59c900f..994f9c1245 100644
--- a/core/cortex-m/timer.c
+++ b/core/cortex-m/timer.c
@@ -138,6 +138,13 @@ int timer_cancel(task_id_t tskid)
void usleep(unsigned us)
{
uint32_t evt = 0;
+
+ /* If task scheduling has not started, just delay */
+ if (!task_start_called()) {
+ udelay(us);
+ return;
+ }
+
ASSERT(us);
do {
evt |= task_wait_event(us);
diff --git a/include/task.h b/include/task.h
index 660195460e..075ed43698 100644
--- a/include/task.h
+++ b/include/task.h
@@ -99,22 +99,26 @@ void task_start_irq_handler(void *excep_return);
* last call of the interrupt handler. */
void task_resched_if_needed(void *excep_return);
-/* Initializes tasks and interrupt controller. */
+/* Initialize tasks and interrupt controller. */
int task_pre_init(void);
/* Starts task scheduling. Does not normally return. */
int task_start(void);
-/* Enables an interrupt. */
+/* Return non-zero if task_start() has been called and task scheduling has
+ * started. */
+int task_start_called(void);
+
+/* Enable an interrupt. */
void task_enable_irq(int irq);
-/* Disables an interrupt. */
+/* Disable an interrupt. */
void task_disable_irq(int irq);
-/* Software-triggers an interrupt. */
+/* Software-trigger an interrupt. */
void task_trigger_irq(int irq);
-/* Clears a pending interrupt.
+/* Clear a pending interrupt.
*
* Note that most interrupts can be removed from the pending state simply by
* handling whatever caused the interrupt in the first place. This only needs
@@ -99,22 +99,26 @@ void task_start_irq_handler(void *excep_return);
int task_pre_init(void);
-/* Starts task scheduling. Does not normally return. */
+/* Start task scheduling. Does not normally return. */
int task_start(void);